'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _toConsumableArray = require('@babel/runtime/helpers/toConsumableArray');
var compilerDom = require('@vue/compiler-dom');
var parser = require('@babel/parser');
var traverse$1 = require('@babel/traverse');
var generator = require('@babel/generator');
var loaderUtils = require('loader-utils');
var lodash = require('lodash');
var serializeJavascript = require('serialize-javascript');

var genImportDeclaration = function genImportDeclaration(name, path) {
  var normalizePath = path.replace(/\\/g, '/');
  var value = "import ".concat(name, " from '").concat(normalizePath, "'");
  return value;
};
var genMixinImport$1 = function genMixinImport(options) {
  var pathKeys = Object.keys(options).filter(function (key) {
    return key !== 'custom' && typeof options[key] === 'string';
  });
  var mixinImport = {};
  if (!pathKeys.length) {
    return {
      mixinImport: mixinImport,
      mixinImportNodes: []
    };
  }
  pathKeys.forEach(function (key) {
    var mixinKey = "".concat(key, "Mixin");
    mixinImport[mixinKey] = genImportDeclaration(mixinKey, options[key]);
  });
  var mixinImportAst = parser.parse(Object.values(mixinImport).join('\n'), {
    sourceType: 'module'
  });
  var mixinImportNodes = mixinImportAst.program.body;
  return {
    mixinImport: mixinImport,
    mixinImportNodes: mixinImportNodes
  };
};
var genMixinCustom$1 = function genMixinCustom(options) {
  if (options.custom) {
    var customMixinAst = parser.parse("const customMixin = ".concat(options.custom), {
      sourceType: 'module'
    });
    return customMixinAst.program.body;
  }
  return [];
};
var genNewMixinNode$1 = function genNewMixinNode() {
  var ast = parser.parse("const customMixin = { mixins: [] }", {
    sourceType: 'module'
  });
  return ast.program.body[0].declarations[0].init.properties[0];
};
var generate$1 = {
  genMixinImport: genMixinImport$1,
  genMixinCustom: genMixinCustom$1,
  genNewMixinNode: genNewMixinNode$1
};

var toString = function toString(val) {
  return Object.prototype.toString.call(val);
};
var isFunction = function isFunction(val) {
  return toString(val).slice(8, -1) === 'Function';
};
var isObject = function isObject(val) {
  return toString(val).slice(8, -1) === 'Object';
};
var isConstructorFn = function isConstructorFn(val) {
  var PropTypeStr = ['function String() { [native code] }', 'function Number() { [native code] }', 'function Boolean() { [native code] }', 'function Array() { [native code] }', 'function Object() { [native code] }', 'function Date() { [native code] }', 'function Function() { [native code] }', 'function Symbol() { [native code] }'];
  var validate = function validate(val) {
    return isFunction(val) && PropTypeStr.includes(val.toString());
  };
  var genConstructorStr = function genConstructorStr(val) {
    return val.toString().match(/^function\s([^(]+)/g)[0].slice(9);
  };
  if (Array.isArray(val)) {
    if (val.every(validate)) {
      return "__ConstructorFn([".concat(val.map(genConstructorStr).join(', '), "])");
    }
  } else {
    if (validate(val)) {
      return "__ConstructorFn(".concat(genConstructorStr(val), ")");
    }
  }
  return;
};
var removeConstructorFnTag = function removeConstructorFnTag(str) {
  return str.replace(/['"]__ConstructorFn\(([^)]+)\)['"]/g, '$1');
};

// 移除属性名两端的双引号
var removePropQuotes = function removePropQuotes(str) {
  return str.replace(/"([\w\d_]+)":/g, '$1:');
};
var stringify_1$1 = function stringify_1(obj) {
  var handleConstructorVal = function handleConstructorVal(obj) {
    var str;
    Object.keys(obj).forEach(function (key) {
      if (str = isConstructorFn(obj[key])) {
        obj[key] = str;
      }
      if (isObject(obj[key])) {
        handleConstructorVal(obj[key]);
      }
    });
  };
  handleConstructorVal(obj);
  var res = serializeJavascript(obj, {
    space: 2
  });
  return removePropQuotes(removeConstructorFnTag(res));
};
var stringify$1 = {
  stringify: stringify_1$1
};

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
var validate = function validate(rule, targetStr) {
  var pattern = new RegExp(rule, 'g');
  return pattern.test(targetStr);
};

// 获取 vue 文件的路径，并将反斜杠转为斜杠
var getFilePath = function getFilePath(str) {
  var start = str.indexOf('!');
  var end = str.indexOf('.vue', start);
  var path = str.slice(start + 1, end + 4);
  return path.replace(/\\/g, '/');
};

/**
 * 校验当前文件是否排除
 * @param {string|string[]} rules 排除的规则
 * @param {string} currentRequest 当前文件路径
 * @returns {boolean}
 */
var validateExclude_1 = function validateExclude_1(rules, currentRequest) {
  if (!rules) {
    return false;
  }
  if (typeof rules === 'string') {
    rules = [rules];
  }
  var filePath = getFilePath(currentRequest);
  var _iterator = _createForOfIteratorHelper(rules),
    _step;
  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var rule = _step.value;
      if (validate(rule, filePath)) {
        return true;
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
  return false;
};
var validateExclude$1 = {
  validateExclude: validateExclude_1
};

var parse = compilerDom.parse;
var traverse = traverse$1["default"];
var generate = generator["default"];
var getOptions = loaderUtils.getOptions,
  getCurrentRequest = loaderUtils.getCurrentRequest;
var omit = lodash.omit;
var genMixinCustom = generate$1.genMixinCustom,
  genMixinImport = generate$1.genMixinImport,
  genNewMixinNode = generate$1.genNewMixinNode;
var stringify = stringify$1.stringify;
var validateExclude = validateExclude$1.validateExclude;
var handler = function handler(source, sourceMaps) {
  var _scriptAst$program$bo;
  var originOptions = getOptions(this);
  var exclude = originOptions.exclude;
  var currentRequest = getCurrentRequest(this);
  // 校验是否排除当前文件
  if (validateExclude(exclude, currentRequest)) {
    return source;
  }

  // 排除 exclude 属性
  var options = omit(originOptions, 'exclude');
  if (!Object.keys(options).length) return source;
  var sourceAST = parse(source);
  var script = sourceAST.children.find(function (v) {
    return v.tag === 'script';
  });
  var _ref = [script.loc.start.offset, script.loc.end.offset],
    startTag = _ref[0],
    endTag = _ref[1];
  var scriptTag = source.slice(startTag, endTag);
  var scriptContent = scriptTag.replace(/^<script>|<\/script>$/g, '');
  var start = startTag + '<script>'.length;
  var end = endTag - '</script>'.length;

  // custom 配置
  var customMixinNodes = genMixinCustom(options);
  // import mixin 配置
  var mixinImportInfo = genMixinImport(options);
  var scriptAst = parser.parse(scriptContent, {
    plugins: ['jsx'],
    sourceType: 'module'
  });
  var newImportNodes = mixinImportInfo.mixinImportNodes;
  traverse(scriptAst, {
    ImportDeclaration: function ImportDeclaration(path) {
      // 收集所有的 import 节点
      scriptAst.program.body = scriptAst.program.body.filter(function (node) {
        return node !== path.node;
      });
      newImportNodes.push(path.node);
    },
    ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
      var _mixins$value$element;
      var properties = path.node.declaration.properties;
      // 既没有 customMixin 也没有从外部引入 importMixin
      if (!customMixinNodes.length && !mixinImportInfo.mixinImportNodes.length) return;
      var mixins = properties.find(function (property) {
        return property.key.name === 'mixins';
      });
      // 是否有 mixins 配置
      if (!mixins) {
        mixins = genNewMixinNode();
        properties.unshift(mixins);
      }
      var newMixins = Object.keys(mixinImportInfo.mixinImport);
      if (customMixinNodes.length) {
        newMixins.push('customMixin');
      }
      var genMixinsElements = newMixins.map(function (key) {
        return {
          type: 'Identifier',
          name: key
        };
      });
      (_mixins$value$element = mixins.value.elements).push.apply(_mixins$value$element, _toConsumableArray(genMixinsElements));
    }
  });
  newImportNodes.push.apply(newImportNodes, _toConsumableArray(customMixinNodes));
  // 插入 mixin 节点
  (_scriptAst$program$bo = scriptAst.program.body).unshift.apply(_scriptAst$program$bo, _toConsumableArray(newImportNodes));
  var newScript = generate(scriptAst).code;
  var newSource = source.slice(0, start) + "\n".concat(newScript, "\n") + source.slice(end);
  return newSource;
};
var src = handler;
var stringify_1 = stringify;
src.stringify = stringify_1;

exports.default = src;
exports.stringify = stringify_1;
